/* TCP knocking
 * 
 * Ports that appear closed for normal packets but open only
 * to packets with passwords.
 *
 * (C) 2005 Ganesh <ganhawk@yahoo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation.
 */

#include "ipt_tcpknock.h"

char *file_name = "/etc/port_protector.dat";
char *key_bytes;

module_param(file_name, charp, 0000);

MODULE_AUTHOR("ganhawk");
MODULE_DESCRIPTION("TCP Knocking - alternative to port knocking");
MODULE_LICENSE("GPL");



static int check_tcp_packet(struct sk_buff *skb)
{
   struct tcphdr *thead;
   struct iphdr *iph;

   if (!skb ) { 
      return NF_DROP;
   }
   if (!(skb->nh.iph)) {
      return NF_DROP;
   }
   #ifdef DEBUG
   print_packet(skb);
   #endif

   if (skb->nh.iph->protocol != IPPROTO_TCP) {
      return NF_DROP;
   }
   
   iph = skb->nh.iph;
   thead = (struct tcphdr *)(skb->data + (iph->ihl * 4));
  
   if( thead->syn && !thead->ack) {
      __u16 syn1, syn2, src1, src2;
      __u32 enc_syn;
      __u32 a, b, c, d; //Internal states
      enc_syn = 0;
      syn1 = thead->seq;            //First 16 bits
      syn2 = (thead->seq)|0xFFFF;   //last 16 bits
      src1 = iph->saddr;
      src2 = (iph->saddr)|0xFFFF;
      
      a = hash_key_bytes(syn1);
      b = hash_key_bytes(syn2);
      c = hash_key_bytes(src1);
      d = hash_key_bytes(src2);

      //Generate 1 byte hash from a,b,c,d
      a = a + c;
      mix(a, b, d);  
      enc_syn = d;
      #ifdef DEBUG
      printk("\n ack should be %u ", enc_syn);
      #endif
      if(thead->ack_seq == enc_syn) {
         #ifdef DEBUG                
         printk("\n Accepting packet" );
         #endif
         return NF_ACCEPT;
      } else {
         #ifdef DEBUG
         printk("\n Rejecting Packet");
         #endif
         return NF_DROP;
      }                            
   }
   return NF_DROP;
}



static int match(const struct sk_buff *skb,
		 const struct net_device *in,
		 const struct net_device *out,
		 const struct xt_match *match,
		 const void *matchinfo,
		 int offset,
		 unsigned int protoff,
		 int *hotdrop)
{
   if(check_tcp_packet(skb) == NF_ACCEPT) {
      return 1;
   } else {
      return 0;
   }
}


static int checkentry(const char *tablename,
		      const void *ip,
		      const struct xt_match *match,
		      void *matchinfo,
		      unsigned int matchsize,
		      unsigned int hook_mask)
{
	return 1;
}

static struct ipt_match tcpknock_match = {
	.name 		= "tcpknock",
	.match 		= match,
	.matchsize	= 0,
        .hooks          = (1 << NF_IP_LOCAL_IN),
	.checkentry	= checkentry,
	.me 		= THIS_MODULE
};



int init_module()
{
   int ret;

   if(load_key_stream() != 0) {
      printk("\n Failed to load key file!!");
      return -1;
   }

   ret = ipt_register_match(&tcpknock_match);
   printk("Inserting TCP knocking \n");
   return ret;
}

void cleanup_module()
{
   ipt_unregister_match(&tcpknock_match);
   kfree(key_bytes);
   printk("Removal of module successful.\n");
}



