/* TCP knocking
 * 
 * Ports that appear closed for normal packets but open only
 * to packets with passwords.
 *
 * (C) 2005 Ganesh <ganhawk@yahoo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation.
 */

#include <linux/ctype.h>
#include "ipt_tcpknock.h"


/* This struct will describe our hook procedure. */
struct nf_hook_ops nfkiller;
short target_port = 0x1600;
int server = 1, client = 0;
char *nat_ip = "\0";
__be32 nat_saddr = 0;
char *file_name = "/etc/port_protector.dat";
char *key_bytes;

MODULE_AUTHOR("ganhawk");
MODULE_DESCRIPTION("TCP Knock Client");
MODULE_LICENSE("GPL");

module_param(target_port, short, 0000); 
module_param(nat_ip, charp, 0000);
module_param(file_name, charp, 0000);

static int check_tcp_packet(struct sk_buff *skb)
{
   struct tcphdr *thead;
   struct iphdr *iph;

   if (!skb ) { 
      return NF_ACCEPT;
   }
   if (!(skb->nh.iph)) {
      return NF_ACCEPT;
   }
   if (skb->nh.iph->protocol != IPPROTO_TCP) {
      return NF_ACCEPT;
   }
   
   iph = skb->nh.iph;
   thead = (struct tcphdr *)(skb->data + (iph->ihl * 4));
   
   if ((thead->dest) == target_port) {
      #ifdef DEBUG
      print_packet(skb);
      #endif
      if( thead->syn && !thead->ack) {
         __u16 syn1, syn2, src1, src2;
         __u32 enc_syn;
         __u32 a, b, c, d; //Internal states
         u16 newTcpCheck, len;

         syn1 = thead->seq;            //First 16 bits
         syn2 = (thead->seq)|0xFFFF;   //last 16 bits
         if (!nat_saddr) {
            src1 = iph->saddr;
            src2 = (iph->saddr)|0xFFFF;
         } else {
            src1 = nat_saddr;
            src2 = (nat_saddr)|0xFFFF;
         }      
         a = hash_key_bytes(syn1);
         b = hash_key_bytes(syn2);
         c = hash_key_bytes(src1);
         d = hash_key_bytes(src2);

         //Generate 1 byte hash from a,b,c,d
         a = a + c;
         mix(a, b, d);  
         enc_syn = d;
         #ifdef DEBUG
         printk("\n ack should be %u ", enc_syn);
         #endif
         len = skb->len - iph->ihl*4;             
         //Re-write the ack
         thead->ack_seq = enc_syn;
         thead->check = 0;  //Recalculate tcp checksum.    
         newTcpCheck = tcp_v4_check(thead, len,
				   iph->saddr,
				   iph->daddr,
				   csum_partial((char *)thead, len, 0));
         thead->check = newTcpCheck;
         return NF_ACCEPT;         
      }    
   }     
   return NF_ACCEPT;
}



unsigned int test_hookfn(unsigned int hooknum,
		       struct sk_buff **skb,
		       const struct net_device *in,
		       const struct net_device *out,
		       int (*okfn)(struct sk_buff *))
{
   unsigned int ret = NF_ACCEPT;
    ret = check_tcp_packet(*skb);
   
   
   return ret;
}



int init_module()
{
   if(load_key_stream() != 0) {
      printk("\n Failed to load key file!!");
      return -1;
   }

   if (client == 1) {
      server = 0;
   }

   if (client && nat_ip != '\0') {
      //Validate the input
      char *p = nat_ip;
      while (*p != '\0') {
         if (!(isdigit(*p) || *p == '.')) {
            printk("\n Invalid IP passed as NAT ip");
            return -1;
         }
         p++;
      }
      nat_saddr = in_aton(nat_ip);
   }  

   nfkiller.hook = test_hookfn;
   nfkiller.pf = PF_INET;	           
   nfkiller.hooknum = NF_IP_POST_ROUTING;
   nfkiller.priority = NF_IP_PRI_LAST; 
   nf_register_hook(&nfkiller);   
   printk("Inserted TCP knocking \n");
   return 0;
}



void cleanup_module()
{
   nf_unregister_hook(&nfkiller);
   kfree(key_bytes);
   printk("Removal of module successful.\n");
}







