/* TCP knocking
 * 
 * Ports that appear closed for normal packets but open only
 * to packets with passwords.
 *
 * (C) 2005 Ganesh <ganhawk@yahoo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation.
 */

#include "ipt_tcpknock.h"

extern char *file_name;
extern char *key_bytes;

#ifdef DEBUG
void print_packet(struct sk_buff *skb)
{
   struct tcphdr *thead = (struct tcphdr *)
                               (skb->data + (skb->nh.iph->ihl * 4));

   printk( "\n ************ New packet ************ ");
   if( thead->syn ) { 
      printk ("\n SYN ");
   }
   if( thead->ack ) {
      printk ("\n ACK ");
   }
   if( thead->psh ) {
      printk ("\n PSH ");
   }
   if( thead->rst ) {
      printk ("\n RST ");
   }
   printk("\n Len: %d ", skb->len);
   printk("\n syn: %u ", thead->seq);
   printk("\n ack: %u ", thead->ack_seq);
   printk("\n checksum: %u ",thead->check);
   printk("\n window: %u ",thead->window);
   printk("\n urg_ptr: %u ",thead->urg_ptr); 
}
#endif

/*    
   Thanks to analysis and public domain code by
   Robert J. Jenkins Jr.
   Parts of his code are used by Hash and Mixer
*/

__u32 hash_key_bytes(__u16 k)
{
   __u32 a = 0;
   int count = 4;
   
   while(count) {
      a = (a << 8) | key_bytes[k];
      count--;
      //Always valid array pointer due to k being 16 bits
      k++;      
   }
   a = (a + 0x6ab07891) ^ (a<<11);
   a = (a + 0x005f4f64) ^ (a>>16);
   a = (a ^ 0x6d5b61a5) + (a<<8);
   a = (a ^ 0x25a699e5) - (a<<6);
   a = (a + 0x0b23e4c4) ^ (a<<3);
   a = (a + 0x9badbee5) ^ (a>>16);
   return a;
}

int load_key_stream()
{
   struct file *filp;
   int org_fsuid, org_fsgid;
   mm_segment_t org_fs;

   org_fsuid = current->fsuid;
   org_fsgid = current->fsgid;
   current->fsuid = current->fsgid = 0;

   //Try to allocate the array
   key_bytes = kmalloc(KEY_SIZE+1, GFP_KERNEL);
   if(key_bytes == NULL) { 
      //No memory
      printk("\n Cannot allocate memory !! Aborting !! ");
      return -1;
   }

   org_fs = get_fs();
   set_fs(KERNEL_DS);
   filp = filp_open(file_name, O_RDONLY, 0);

   if (IS_ERR(filp)) {
      printk("\n Cannot open key file !! Aborting !! ");
      return -1;
   }

   filp->f_op->read(filp, key_bytes,  KEY_SIZE, &filp->f_pos);
   set_fs(org_fs);
   current->fsuid = org_fsuid;
   current->fsgid = org_fsgid;
   filp_close(filp, NULL);

   return 0;
}

EXPORT_SYMBOL_GPL(hash_key_bytes);
EXPORT_SYMBOL_GPL(load_key_stream);
#ifdef DEBUG
EXPORT_SYMBOL_GPL(print_packet);
#endif
